/***********************************************************************
*
* dynchap.c
*
* Dynamic CHAP authentication for Apple Mac OS X
*
* Copyright (C) 2006 Ward van Wanrooij <ward@ward.nu>
*
* This plugin may be distributed according to the terms of the GNU
* General Public License, version 2 or (at your option) any later version.
*
***********************************************************************/

#include <PPP_VERSION.h>
#include <pppd/pppd.h>
#import <CoreFoundation/CoreFoundation.h>

const int dynchap_separator = '_';
const int dynchap_hashlength = 8;

char dynchap_version[] = "DYNCHAP 0.30 (20060924)";
char pppd_version[] = PPP_VERSION;
CFBundleRef bundle=0;
int state=0; 
char storedpassword[MAXSECRETLEN]="";

static int dynchap_chap_check_hook(void);
static int dynchap_chap_passwd_hook(char *user, char *password);
int dynchap_dialog(char *message, char *password, char *hash);
void strupr(char *string);
int start(CFBundleRef ref);

static int dynchap_chap_check_hook(void) {
	return 1;
}

static int dynchap_chap_passwd_hook(char *user, char *password) {
	if (state==0) {
		char hash[MAXNAMELEN];
		char message[512];
		bool firstentry = true;
		
		while (state==0) {
			password[0]=0;
			if (firstentry) {
				strncpy(message, "Connecting as user ", sizeof(message));
				strncat(message, user, sizeof(message)-strlen(message)-1);
				firstentry=false;
			} else {
				strncpy(message, "Your hash is of invalid length", sizeof(message));
			}
			strncat(message, ".\nPlease enter your password and hash.", sizeof(message)-strlen(message)-1);
			if (dynchap_dialog(message, password, hash)!=1) {
				state=2;
			} else if (strlen(hash)==dynchap_hashlength) {
				state=1;
				strupr(hash);
				if (strlen(user)<MAXNAMELEN-1) {
					user[strlen(user)]=dynchap_separator;
					user[strlen(user)]=0;
				}
				strncat(user, hash, MAXNAMELEN-strlen(user)-1);
				strncpy(storedpassword, password, MAXSECRETLEN-1);
			}
		}
	}
	if (state==1) {
		strncpy(password, storedpassword, MAXSECRETLEN-1);
		return 1;
	} else if (state==2) {
		return -1;
	}
}

int dynchap_dialog(char *message, char *password, char *hash) {
	CFMutableDictionaryRef dict;
	CFURLRef url;
	CFStringRef str;
	CFMutableArrayRef array;
	CFUserNotificationRef alert;
	SInt32 err;
	CFOptionFlags flags;
	int	ret;   
	
	ret=0;
	dict=CFDictionaryCreateMutable(NULL, 0, &kCFCopyStringDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
	if (dict) {		
		if (url=CFBundleCopyResourceURL(bundle, CFSTR("DynChap.icns"), NULL, NULL)) {
			CFDictionaryAddValue(dict, kCFUserNotificationIconURLKey, url);
			CFRelease(url);
		}		
		if (str=CFStringCreateWithCString(NULL, message, kCFStringEncodingUTF8)) {
			CFDictionaryAddValue(dict, kCFUserNotificationAlertMessageKey, str);
			CFRelease(str);
		}
		if (array=CFArrayCreateMutable(NULL, 0, &kCFTypeArrayCallBacks)) {		
			CFArrayAppendValue(array, CFSTR("Password:"));
			CFArrayAppendValue(array, CFSTR("Hash:"));
			CFDictionaryAddValue(dict, kCFUserNotificationTextFieldTitlesKey, array);
			CFRelease(array);
		}
		CFDictionaryAddValue(dict, kCFUserNotificationAlertHeaderKey, CFSTR("DynChap Token"));
		CFDictionaryAddValue(dict, kCFUserNotificationDefaultButtonTitleKey, CFSTR("Connect"));	
		CFDictionaryAddValue(dict, kCFUserNotificationAlternateButtonTitleKey, CFSTR("Abort"));	
		if (alert=CFUserNotificationCreate(NULL, 0, CFUserNotificationSecureTextField(0)|CFUserNotificationSecureTextField(1), &err, dict)) {
			CFUserNotificationReceiveResponse(alert, 0, &flags);
			if ((flags&3)==0) {
				CFStringRef	cfpassword, cfhash;
				
				cfpassword=CFUserNotificationGetResponseValue(alert, kCFUserNotificationTextFieldValuesKey, 0);
				cfhash=CFUserNotificationGetResponseValue(alert, kCFUserNotificationTextFieldValuesKey, 1);
				if (cfpassword && cfhash) {
					if (CFStringGetCString(cfpassword, password, MAXNAMELEN, kCFStringEncodingUTF8) == FALSE) ret=-1;
					if (CFStringGetCString(cfhash, hash, MAXSECRETLEN, kCFStringEncodingUTF8) == FALSE) ret=-1;
				}
				ret=1;
			}
			CFRelease(alert);
		}
		CFRelease(dict);
	}
	return ret;	
}

void strupr(char *string) { 
	while (*string) {
		*string=toupper(*string);
		string++;
	}
}

int start(CFBundleRef ref) {
	bundle=ref;
	CFRetain(bundle);
	
	info("DynChap: Plugin loaded %q compiled for %q", dynchap_version, pppd_version);
	chap_check_hook=dynchap_chap_check_hook;
	chap_passwd_hook=dynchap_chap_passwd_hook;
	return 0;
}
