(*

DynChapDialer

Copyright (C) 2004 Ward van Wanrooij <ward@ward.nu>.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
   3. The name of the author may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*)

unit udcd1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, ExtCtrls, Ras;

type
  TfrmMain = class(TForm)
    btnConnect: TButton;
    edtUsername: TEdit;
    lblUsername: TLabel;
    lblPassword: TLabel;
    lblHash: TLabel;
    edtPassword: TEdit;
    cbxConnection: TComboBox;
    edtHash: TEdit;
    bvl2: TBevel;
    btnCancel: TButton;
    bvl3: TBevel;
    lblExplanation: TLabel;
    imgConnection: TImage;
    bvl1: TBevel;
    procedure btnConnectClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure LoadConnection;
    procedure FlipControls(Enable: Boolean);
    procedure cbxConnectionChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure imgConnectionClick(Sender: TObject);
  private
    Connecting: Boolean;
    Conn: THRasConn;
    FileName: String;
    Connection: String;
    Separator: String;
    HashLength: Integer;
  public
    { Public declarations }
  end;

procedure RasCallback(unMsg: UINT; rasconnstate: TRasConnState; dwError: DWORD); stdcall;
function RasExplanation(r: TRasConnState): String;

var
  frmMain: TfrmMain;
const
  dynchap_version = 'DYNCHAP 0.2 (20041123)';

implementation

{$R *.DFM}

uses RasError, LmCons, IniFiles;

procedure RasCallback(unMsg: UINT; rasconnstate: TRasConnState; dwError: DWORD); stdcall;
var
  S: String;
begin
  if (rasconnstate=RASCS_AuthNotify) and (dwError>0) then
  begin
    frmMain.lblExplanation.Caption:='Access denied.';
    MessageBox(frmMain.Handle, 'Access denied because username, password and/or hash are invalid.', 'Error', MB_ICONERROR+MB_OK);
    frmMain.FlipControls(True);
  end
  else if (rasconnstate and RASCS_DONE)=RASCS_DONE then
  begin
    PostMessage(frmMain.Handle, WM_CLOSE, 0, 0);
  end
  else
  begin
    S:=RasExplanation(rasconnstate);
    if S<>'' then frmMain.lblExplanation.Caption:=S;
    //if S='' then MessageBox(0, PChar(IntToStr(rasconnstate)), '', 0);
    if dwError>0 then frmMain.FlipControls(True);
  end;
end;

function RasExplanation(r: TRasConnState): String;
begin
  case r of
    RASCS_OpenPort: Result:='Opening communication port.';
    RASCS_PortOpened: Result:='Communication port opened successfully.';
    RASCS_ConnectDevice: Result:='Connecting device.';
    RASCS_DeviceConnected: Result:='Device connected successfully.';
    RASCS_AllDevicesConnected: Result:='All devices successfully connected.';
    RASCS_Authenticate: Result:='Authenticating.';
    RASCS_AuthProject: Result:='Registering on the network.';
    RASCS_AuthLinkSpeed: Result:='Calculating link-speed.';
    RASCS_StartAuthentication: Result:='Authenticating.';
    RASCS_LogonNetwork: Result:='Logging on to the network.';
    RASCS_Disconnected: Result:='Disconnection or failed connection.';
  else
    Result:='';
  end;
  //RASCS_AuthAck: Result:='An authentication request is being acknowledged.';
  //RASCS_Authenticated: Result:='Authenticated.';
end;

procedure TfrmMain.btnConnectClick(Sender: TObject);
var
  Extensions: TRasDialExtensions;
  Params: TRasDialParams;
  Pass: BOOL;
  Err: DWORD;
begin
  if Length(edtHash.Text)<>HashLength then
  begin
    Application.MessageBox('Your hash is of invalid length.'#13#10'Please try again.', 'Error', MB_ICONERROR+MB_OK);
    Exit;
  end;
  FillChar(Params, SizeOf(Params), 0);
  Params.dwSize:=SizeOf(Params);
  StrPCopy(Params.szEntryName, Connection);
  Err:=RasGetEntryDialParams(nil, Params, Pass);
  if Err=0 then
  begin
    Conn:=0;
    StrPCopy(Params.szUserName, Copy(edtUsername.Text+Separator+UpperCase(edtHash.Text), 1, UNLEN-1));
    StrPCopy(Params.szPassword, Copy(edtPassword.Text, 1, PWLEN-1));
    FillChar(Extensions, SizeOf(Extensions), 0);
    Extensions.dwSize:=SizeOf(Extensions);
    Extensions.dwfOptions:=0;
    FlipControls(False);
    Err:=RasDial(@Extensions, nil, @Params, 0, @RasCallback, Conn);
    if Err>0 then
      Application.MessageBox(PChar(Format('During connecting error %d occurred.'#13#10'Please check your settings.', [Err])), 'Error', MB_ICONERROR+MB_OK);
  end
  else
    Application.MessageBox(PChar(Format('During setup error %d occurred.'#13#10'Please check your settings.', [Err])), 'Error', MB_ICONERROR+MB_OK);
end;

procedure TfrmMain.btnCancelClick(Sender: TObject);
var
  Sts: TRasConnStatusA;
begin
  if Connecting then
  begin
    btnCancel.Enabled:=False;
    lblExplanation.Caption:='Disconnecting.';
    RasHangup(Conn);
    FillChar(Sts, SizeOf(Sts), 0);
    Sts.dwSize:=SizeOf(Sts);
    while RasGetConnectStatus(Conn, Sts)<>ERROR_INVALID_HANDLE do
    begin
      Application.ProcessMessages;
      Sleep(10);
    end;
    lblExplanation.Caption:='Disconnected.';
    btnCancel.Enabled:=True;
    FlipControls(True);
  end
  else
    Close;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  L: TStringList;
  Ini: TIniFile;
begin
  FileName:=ExtractFileDir(ParamStr(0))+'\dynchapd.ini';
  L:=TStringList.Create;
  Ini:=TIniFile.Create(FileName);
  Ini.ReadSections(L);
  cbxConnection.Items.Assign(L);
  Ini.Free;
  L.Free;
  cbxConnection.ItemIndex:=0;
  if cbxConnection.Items.Count=0 then
  begin
    Application.MessageBox(PChar(FileName+' does not exist. Closing dialer.'), 'Error', MB_ICONERROR+MB_OK);
    Close;
  end;
  LoadConnection;
  if Screen.Fonts.IndexOf('Wingdings')>0 then
  begin
    edtPassword.Font.Name:='Wingdings';
    edtPassword.PasswordChar:='l';
    edtHash.Font.Name:='Wingdings';
    edtHash.PasswordChar:='l';
  end;
  FlipControls(True);
end;

procedure TfrmMain.cbxConnectionChange(Sender: TObject);
begin
  LoadConnection;
end;

procedure TfrmMain.LoadConnection;
var
  Ini: TIniFile;
  S: String;
begin
  S:=cbxConnection.Items[cbxConnection.ItemIndex];
  Ini:=TIniFile.Create(FileName);
  edtUserName.Text:=Ini.ReadString(S, 'username', 'user');
  Separator:=Ini.ReadString(S, 'separator', '_');
  Connection:=Ini.ReadString(S, 'connection', 'vpn');
  HashLength:=Ini.ReadInteger(S, 'hashlength', 8);
  Ini.Free;
  edtHash.MaxLength:=HashLength;
  edtPassword.Text:='';
  edtHash.Text:=''
end;

procedure TfrmMain.FlipControls(Enable: Boolean);
begin
  cbxConnection.Enabled:=Enable;
  edtUsername.Enabled:=Enable;
  edtPassword.Enabled:=Enable;
  edtHash.Enabled:=Enable;
  btnConnect.Enabled:=Enable;
  Connecting:=not Enable;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  if Length(edtUsername.Text)>0 then
    ActiveControl:=edtPassword
  else
    ActiveControl:=edtUsername;
end;

procedure TfrmMain.imgConnectionClick(Sender: TObject);
begin
  Application.MessageBox(PChar('Dynchap Dialer'#13#10+dynchap_version+#13#10'Copyright (C) 2004 Ward van Wanrooij <ward@ward.nu>'#13#10'For more information visit http://www.ward.nu/computer/dynchap'), 'About', MB_ICONINFORMATION+MB_OK);
end;

end.
