/*

DynChapToken

Copyright (C) 2004 Ward van Wanrooij <ward@ward.nu>.
   
Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
   3. The name of the author may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

import fi.iki.santtu.md5.MD5;
import java.util.Calendar;
import java.util.TimeZone;
import java.util.Date;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;

public class DynChap extends MIDlet {
	//key:	Private key used to XOR the serial before storing in recordstore.
	//		Change this key to whatever you like, as long as it contains seriallength elements.
	//		The key is XOR'ed to prevent someone from installing a midlet over this midlet, and thus gaining access to the serial.
	//		Now, the attacker also needs the original JAR (for the key).
	private static byte[] key={1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32};
	//seriallength: Must match dynchap_seriallength
	private static int seriallength=32;
	private static final String dynchap_version = "DYNCHAP 0.22 (20041204)";
	private Display display;
	private Displayable curdisp=null;
	private boolean firstTime;
	private byte[] serial=null;
	private int timediff=0;
	private int screenWidth=0;
	private int screenHeight=0;
	private int numColors=0;
	public DynChapMain _myDynChapMain=null;
	public DynChapSerial _myDynChapSerial=null;
	public DynChapTimediff _myDynChapTimediff=null;
	
	public DynChap() {
		display=Display.getDisplay(this);
		firstTime=true;
	}

	public void startApp() {
		if (firstTime) {
			RecordStore rs;
			RecordEnumeration re;
			
			firstTime=false;
			DummyCanvas dummy=new DummyCanvas();
			show(dummy);
			dummy.repaint();
			try {
				byte[] in;
				byte[] buffer=new byte[seriallength];
				buffer[0]='0';
				rs=RecordStore.openRecordStore("dynchap", true);
				while (rs.getNumRecords()<2) {
					rs.addRecord(buffer, 0, 1);
				}
				in=rs.getRecord(1);
				if ((in==null) || (in.length<seriallength)) {
					serial=null;
				} else {
					for (int i=0; i<seriallength; i++) {
						buffer[i]=(byte)(in[i]^key[i]);
					}
					serial=buffer;
				}
				in=rs.getRecord(2);
				if ((in==null) || (in.length==0)) {
					timediff=0;
				} else {
					try {
						timediff=Integer.parseInt(new String(in));
					} catch (NumberFormatException e) {
						timediff=0;
					}
				}		
				rs.closeRecordStore();
				if (serial==null) {
					show(myDynChapSerial());
				} else {
					show(myDynChapMain());
				}
			} catch (Exception e) {
				serial=null;
				error(e.toString(), myDynChapMain());
			}
		} else {
			display.setCurrent(curdisp);
		}
	}

	public void pauseApp() {
	}

	public void destroyApp(boolean unconditional) {
	}

	public void show(Displayable disp) {
		curdisp=disp;
		display.setCurrent(disp);
	}
	
	public void error(String msg, Displayable next) {
		final Alert a=new Alert("Error", msg, null, AlertType.ERROR);
		a.setTimeout(a.FOREVER);
		display.setCurrent(a, next);
		System.err.println("Error: "+msg);
	}

	public DynChapMain myDynChapMain() {
		if (_myDynChapMain==null) {
			_myDynChapMain=new DynChapMain(this);
		}
		return _myDynChapMain;
	}

	public DynChapSerial myDynChapSerial() {
		if (_myDynChapSerial==null) {
			_myDynChapSerial=new DynChapSerial(this);
		}
		return _myDynChapSerial;
	}

	public DynChapTimediff myDynChapTimediff() {
		if (_myDynChapTimediff==null) {
			_myDynChapTimediff=new DynChapTimediff(this);
		}
		return _myDynChapTimediff;
	}

	class DummyCanvas extends Canvas {
		protected void paint(Graphics g){
			screenHeight=getHeight();
			screenWidth=getWidth();
			numColors=display.numColors();
		}
	}
    
	class DynChapMain extends Form implements CommandListener {
		private Command mainExitCommand=new Command("Exit", Command.EXIT, 1);
		private Command mainBackCommand=new Command("Exit", Command.BACK, 1);
		private Command mainRefreshCommand=new Command("Refresh", Command.ITEM, 1);
		private Command mainSerialCommand=new Command("Enter serial", Command.ITEM, 2);
		private Command mainTimediffCommand=new Command("Enter timediff", Command.ITEM, 3);
		private DynChap parent;
		public int figure[][] = {
	{8184, 16380, 14364, 28718, 28718, 28750, 28750, 28814, 28942, 28942, 29198, 29198, 15388, 16380, 8184, 0},
	{384, 896, 1920, 3968, 8064, 896, 896, 896, 896, 896, 896, 896, 896, 8176, 8176, 0},
	{2040, 8190, 15375, 6151, 14, 28, 56, 112, 224, 448, 896, 1792, 3584, 8191, 16383, 0},
	{4080, 8184, 14364, 28, 28, 56, 2032, 2032, 56, 28, 28, 28, 14364, 8184, 4080, 0},
	{112, 240, 496, 880, 1648, 3184, 6256, 14448, 32767, 32767, 112, 112, 112, 112, 112, 0},
	{32766, 32767, 28672, 28672, 28672, 28672, 32760, 32764, 28, 14, 14, 14, 28, 32764, 32760, 0},
	{112, 224, 448, 896, 1792, 3584, 8160, 16376, 14364, 14364, 14364, 14364, 14364, 8184, 4064, 0},
	{32767, 32767, 7, 14, 28, 56, 112, 2044, 2044, 896, 1792, 3584, 7168, 14336, 28672, 0},
	{4080, 8184, 14364, 14364, 14364, 14364, 8184, 16380, 14364, 14364, 14364, 14364, 14364, 8184, 4080, 0},
	{4064, 8184, 14364, 14364, 14364, 14364, 14364, 16376, 4080, 112, 224, 448, 896, 1792, 3584, 0},
	{448, 992, 1904, 1904, 3640, 3640, 7196, 8188, 8188, 14350, 14350, 14350, 28679, 28679, 28679, 0},
	{32764, 32766, 28679, 28679, 28679, 28686, 32764, 32764, 28686, 28679, 28679, 28679, 28679, 32766, 32764, 0},
	{4092, 8190, 15374, 14342, 14336, 28672, 28672, 28672, 28672, 28672, 14336, 14342, 15374, 8190, 4092, 0},
	{32760, 32764, 28686, 28679, 28679, 28679, 28679, 28679, 28679, 28679, 28679, 28679, 28679, 32764, 32760, 0},
	{32767, 32767, 28672, 28672, 28672, 28672, 32766, 32766, 28672, 28672, 28672, 28672, 28672, 32767, 32767, 0},
	{32767, 32767, 28672, 28672, 28672, 28672, 32766, 32766, 28672, 28672, 28672, 28672, 28672, 28672, 28672, 0}
			};
		
		public DynChapMain(DynChap parent) {
			super("DynChap Token");
			this.parent=parent;
			this.addCommand(mainExitCommand);
			this.addCommand(mainBackCommand);
			this.addCommand(mainRefreshCommand);
			this.addCommand(mainSerialCommand);
			this.addCommand(mainTimediffCommand);
			this.setCommandListener(this); 
			showHash();
		}

		public void showHash() {
			StringBuffer message;	
			Calendar cal;
			MD5 md5;
			byte[] hash;

			while (this.size()>0)
				this.delete(0);	
			if (parent.serial==null) {
				this.append(new StringItem("Serial key missing", ""));
			} else {
				message=new StringBuffer(32);
				message.append(new String(serial));
				cal=Calendar.getInstance();
				if (parent.timediff==0) {
					cal.setTimeZone(TimeZone.getTimeZone("GMT"));
				} else {
					Date dat=cal.getTime();
					dat.setTime(dat.getTime()+parent.timediff*60000);
					cal.setTime(dat);
				}
				message.append(fmt(cal.get(cal.YEAR), 4, 10));
				message.append(fmt(cal.get(cal.MONTH)+1, 2, 10));
				message.append(fmt(cal.get(cal.DATE), 2, 10));
				message.append(fmt(cal.get(cal.HOUR_OF_DAY), 2, 10));
				message.append(fmt(cal.get(cal.MINUTE), 2, 10));
				//System.err.println(message.toString());
				
				md5=new MD5();
				md5.Init();
				md5.Update(message.toString());
				hash=md5.Final();			   
				int[] hash2={(hash[15]>>4)&0x0F, hash[15]&0x0F, (hash[11]>>4)&0x0F, hash[11]&0x0F, (hash[7]>>4)&0x0F, hash[7]&0x0F, (hash[3]>>4)&0x0F, hash[3]&0x0F};
				if (screenWidth<=128) {
					this.append(new StringItem("Hash", fmt(hash[15]&0xFF, 2, 16)+fmt(hash[11]&0xFF, 2, 16)+fmt(hash[7]&0xFF, 2, 16)+fmt(hash[3]&0xFF, 2, 16)));
				} else {
					Image img=Image.createImage(128, 16);
					Graphics g=img.getGraphics();
					g.setColor(0, 0, 0);
					for (int k=0; k<8; k++) {
						for (int i=0; i<16; i++) {
							for (int j=0; j<16; j++) {
								if (((figure[hash2[k]][i]>>(15-j))&1)==1) {
									g.drawLine(k*16+j, i, k*16+j, i);
								}
							}
						}
					}
					this.append(new ImageItem("", img, ImageItem.LAYOUT_LEFT, fmt(hash[15]&0xFF, 2, 16)+fmt(hash[11]&0xFF, 2, 16)+fmt(hash[7]&0xFF, 2, 16)+fmt(hash[3]&0xFF, 2, 16)));
				}
	       		Runtime.getRuntime().gc();
			}
		}
		
		public void commandAction(Command c, Displayable s) {
			if ((c.equals(mainExitCommand)) || (c.equals(mainBackCommand))) {
				parent.destroyApp(false);
				parent.notifyDestroyed();
			} else if (c.equals(mainSerialCommand)) {
				parent.show(parent.myDynChapSerial());
			} else if (c.equals(mainTimediffCommand)) {
				parent.show(parent.myDynChapTimediff());
			} else if (c.equals(mainRefreshCommand)) {
				showHash();
			}
		}

		String fmt(int value, int precision, int radix) {
			StringBuffer buf=new StringBuffer(precision);
			buf.append(Long.toString(value, radix).toLowerCase());
			for (int i=buf.length(); i<precision; i++) {
				buf.insert(0, "0");
			}
			return(buf.toString());
		}
	}

	class DynChapSerial extends TextBox implements CommandListener {
		private Command serialOKCommand=new Command("OK", Command.OK, 1);
		private Command serialCancelCommand=new Command("Cancel", Command.CANCEL, 1);
		private DynChap parent;

		public DynChapSerial(DynChap parent) {
			super("DynChap Serial Entry", Integer.toString(seriallength), seriallength*2, TextField.ANY);
			this.parent=parent;
			this.addCommand(serialOKCommand);
			this.addCommand(serialCancelCommand);
			this.setCommandListener(this);
			parent.show(this);		
		}
		
		public void commandAction(Command c, Displayable s) {			
			if (c.equals(serialOKCommand)) {
				RecordStore rs;
				RecordEnumeration re;
				String str;
				byte[] in;

				str=this.getString();
				if ((str.length()!=2*seriallength) || (!str.substring(0, str.length()/2).equals(str.substring(str.length()/2))))
					return;
				in=str.substring(0, seriallength).getBytes();			
				try {
					byte[] buffer=new byte[seriallength];
					rs=RecordStore.openRecordStore("dynchap", true);
					for (int i=0; i<seriallength; i++) {
						buffer[i]=(byte)(in[i]^key[i]);
					}
					rs.setRecord(1, buffer, 0, seriallength);
					rs.closeRecordStore();
					parent.serial=in;
					parent.show(myDynChapMain());
					myDynChapMain().showHash();
				} catch (Exception e) {
					parent.serial=null;
					myDynChapMain().showHash();
					parent.error(e.toString(), myDynChapMain());
				}
			} else if (c.equals(serialCancelCommand)) {
				parent.show(myDynChapMain());
			}
		}	
	}

	class DynChapTimediff extends TextBox implements CommandListener {
		private Command timediffOKCommand=new Command("OK", Command.OK, 1);
		private Command timediffCancelCommand=new Command("Cancel", Command.CANCEL, 1);
		private DynChap parent;

		public DynChapTimediff(DynChap parent) {
			super("DynChap Timediff Entry", Integer.toString(parent.timediff), 4, TextField.ANY);
			this.parent=parent;
			this.addCommand(timediffOKCommand);
			this.addCommand(timediffCancelCommand);
			this.setCommandListener(this);
			parent.show(this);		
		}
		
		public void commandAction(Command c, Displayable s) {			
			if (c.equals(timediffOKCommand)) {
				RecordStore rs;
				RecordEnumeration re;
				String str;
				int value;

				str=this.getString();
				try {
					value=Integer.parseInt(str);
				} catch (Exception e) {
					return;
				}
				try {
					rs=RecordStore.openRecordStore("dynchap", true);
					rs.setRecord(2, str.getBytes(), 0, str.length());
					rs.closeRecordStore();
					parent.timediff=value;
					parent.show(myDynChapMain());
					myDynChapMain().showHash();
				} catch (Exception e) {
					parent.timediff=0;
					myDynChapMain().showHash();
					parent.error(e.toString(), myDynChapMain());
				}
			} else if (c.equals(timediffCancelCommand)) {
				parent.show(myDynChapMain());
			}
		}	
	}
}

